"""Unearthed New Command."""
import logging
import os
import shutil

import click
from click import Choice
from unearthed.cli.auth import authorize
from unearthed.cli.util import load_archive, load_public_data
from unearthed.core.models.challenge import CHALLENGE_FILENAME, Challenge

logger = logging.getLogger(__name__)


# Prompt the user to select one of the challenges
# Checks if the user provided a challenge slug and load that without prompt
def _get_challenge(challenge_slug) -> Challenge:
    """Get a challenge instance based on challenge_slug."""
    logger.debug("get_challenge")
    challenges = Challenge.allFromApi()

    # if there are no challenges in list just return None
    if len(challenges) == 0:
        return None

    # check to see if the requested challenge slug is in the list of available
    # challenges
    if challenge_slug is not None:
        challenge = next(
            (challenge for challenge in challenges if challenge.slug == challenge_slug),
            None,
        )
        if challenge is not None:
            return challenge

    # otherwise, ask which challenge template is desired
    click.echo("Available challenges:")
    for idx, challenge in enumerate(challenges):
        click.echo(f" [{idx+1}]  {challenge.name} ({challenge.slug})")

    selected = click.prompt(
        "Which challenge template do you wish to download?",
        type=click.IntRange(min=1, max=len(challenges), clamp=False),
    )
    return challenges[selected - 1]


@click.command()
@click.argument(
    "target_dir",
    type=click.Path(exists=False),
    default=os.getcwd(),
)
@click.option("--challenge", type=click.STRING)
@click.option("--data/--no-data", default=True)
@authorize
def new(target_dir, challenge, data):
    """Create a new Unearthed submission project from an S3 template."""
    logger.debug("new")
    challenge = _get_challenge(challenge)
    if challenge is None:
        return click.echo("You are not registered for any competitions.")

    # note challenge is now a Challenge instance
    logger.debug(f"challenge is {challenge.slug}")

    # Build and validate the path for the template
    # TODO this should really check the folder and fail if it exists
    logger.debug(f"parent_dir is {target_dir}")

    # if the target_dir is the current dir, then create a dir from the
    # challenge slug
    if target_dir == os.getcwd():
        target_dir = os.path.join(target_dir, challenge.slug)
    challenge_file = os.path.join(target_dir, CHALLENGE_FILENAME)

    # check to see if the target_dir is empty of not, and if it is not
    # prompt to overwrite
    if os.path.exists(target_dir) and len(os.listdir(target_dir)) != 0:
        click.echo(f"{target_dir} is not empty")
        if click.confirm("Overwrite?", default=False, abort=True):
            shutil.rmtree(target_dir, ignore_errors=True)

    # load in template from archive
    click.echo("Loading in template...")
    load_archive(challenge.templateUrl, target_dir, challenge.slug + "_template")

    # load in scoring function from archive
    click.echo("Loading in scoring function...")
    load_archive(challenge.scoringFunctionUrl, target_dir, challenge.slug + "_scoring")

    # load in data from archive
    if data:
        click.echo("Loading in data files...")
        load_public_data(challenge, target_dir=target_dir)

    # Save the challenge details for later
    challenge.save(filename=challenge_file)
    click.echo(f"Challenge template downloaded and ready to use at {target_dir}")



"""Unearthed CLI Authentiation."""
import functools
import logging

from unearthed.core.auth.unearthed_auth import UnearthedAuth
from unearthed.core.auth.unearthed_cognito import UnearthedCognito

logger = logging.getLogger(__name__)

# logger.debug("creating cognito session")
cognito_session = UnearthedCognito()

# logger.debug("creating a requests auth handler")
auth_handler = UnearthedAuth(cognito_session)


def authorize(func):
    """Annotation for Click commands to require valid login."""

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        logger.debug(f"checking authorization for {func.__name__} command")
        if cognito_session.is_logged_in():
            return func(*args, **kwargs)
        else:
            print(
                "Command not available, not logged in! Please login via `unearthed login`"
            )

    return wrapper



"""Unearthed Tracker command."""
import json
import os
import webbrowser

import click
from unearthed.cli.errors import bail
from unearthed.cli.config import TRACKER_BASE_URL

curr_dir = os.getcwd()


@click.command()
def tracker():
    """Open Unearthed submission tracker."""

    sub_path = os.path.join(os.getcwd(), ".unearthed_last_submission.json")
    if not os.path.exists(sub_path):
        return click.echo(
            'No submissions found in this directory, try "unearthed submit".'
        )
    with open(sub_path) as json_file:
        sub_res = json.load(json_file)

    # Read in the saved challenge details from when `unearthed new` was run
    challenge_path = os.path.join(os.getcwd(), ".unearthed_challenge.json")
    if not os.path.exists(challenge_path):
        return click.echo("No challenge template found in this directory.")
    try:
        with open(challenge_path) as json_file:
            challenge = json.load(json_file)
    except json.decoder.JSONDecodeError as e:
        bail(
            e,
            "There was a problem decoding the challenge template {}.".format(
                challenge_path
            ),
        )

    # Open Tracker
    tracker_url = "{}/crowdml/tracker/{}".format(
        TRACKER_BASE_URL, sub_res["innovatorPortalSubmissionRef"]
    )
    click.echo(
        "You can track the progress of your last submission at: {}".format(tracker_url)
    )
    webbrowser.open_new_tab(tracker_url)



"""Unearthed specific requests Authentication Helper."""
from requests.auth import AuthBase
from unearthed.core.auth.unearthed_cognito import UnearthedCognito


class UnearthedAuth(AuthBase):
    """Attach the Unearthed token to a given Request object."""

    def __init__(self, cognito_session=None):
        """Attach to, or create, a Cognito session."""
        if not cognito_session:
            self.cognito_session = UnearthedCognito()
        else:
            self.cognito_session = cognito_session

    def __call__(self, r):
        """Add Authorization header and return the request."""
        self.cognito_session.check_token()
        r.headers["Authorization"] = f"Bearer {self.cognito_session.id_token}"
        return r



"""Unearthed Configuration."""
from os import environ

if environ.get("DEBUG") or environ.get("UNEARTHED_DEBUG"):
    LAMBDA_URL = "https://crowdml-dev.unearthed.solutions/"
    TRACKER_BASE_URL = "https://cms.development-q5nzhaa-ailver3isw3iu.au.platformsh.site"
else:
    if environ.get("UNEARTHED_TEST"):
        LAMBDA_URL = "https://crowdml-test.unearthed.solutions/"
        TRACKER_BASE_URL = "https://cms.development-q5nzhaa-ailver3isw3iu.au.platformsh.site"
    else:
        LAMBDA_URL = "https://crowdml.unearthed.solutions/"
        TRACKER_BASE_URL = "https://data-science.unearthed.solutions"



from sys import exit
from traceback import print_exc  # print_tb
from uuid import uuid4

import click
from colorama import Back, Fore, Style


def bail(e, text):
    error_filename = "error-" + str(uuid4())[:8] + ".txt"
    with open(error_filename, "w") as tmp:
        print_exc(file=tmp)
    error_text = getattr(e, "explanation", str(e))
    message = "\n" + Fore.RED + u"\u2718" + " "
    message += text + "\n\n"
    message += Style.RESET_ALL
    message += "The full error has been stored in a file called {} {} {}\n".format(
        Fore.GREEN, error_filename, Style.RESET_ALL
    )
    message += "which you can email to {}{}{} for support.\n".format(
        Fore.GREEN, "support@unearthed.solutions", Style.RESET_ALL
    )
    message += "{}Error: {}{}\n".format(Fore.RED, error_text, Style.RESET_ALL)
    exit(message)


def check_latest_version():
    try:
        from outdated import check_outdated
        from pkg_resources import get_distribution

        dist = get_distribution("unearthed-cli")
        return check_outdated("unearthed-cli", dist.version)
    except Exception:
        return [None, None]


def check_version_and_warn():
    is_outdated, latest_version = check_latest_version()
    if is_outdated:
        message = Fore.YELLOW
        message += "There is a newer version of the unearthed command available.\n"
        message += "Please install it like this: pip install --upgrade unearthed\n"
        message += Style.RESET_ALL
        click.echo(message)



"""Unearthed Logging."""
import os
import logging

logger = logging.getLogger("unearthed")

# enable DEBUG level logging if the UNEARTHED_CLI_DEBUG environment variable
# has been set (to any value)
if os.getenv("UNEARTHED_DEBUG") or os.getenv("UNEARTHED_TEST") is not None:
    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    format = "%(asctime)s:%(levelname)s:%(name)s:%(message)s"
    ch.setFormatter(logging.Formatter(format))
    logger.addHandler(ch)
    # logger.debug('logging initialized')
